package domain

import (
	"errors"
	"fmt"
	"strconv"

	abraFormatter "coopcloud.tech/abra/cli/formatter"
	"coopcloud.tech/abra/cli/internal"
	gandiPkg "coopcloud.tech/abra/pkg/dns/gandi"
	"github.com/libdns/gandi"
	"github.com/sirupsen/logrus"
	"github.com/urfave/cli/v2"
)

// DomainListCommand lists domains.
var DomainListCommand = &cli.Command{
	Name:      "list",
	Usage:     "List domain name records for a zone",
	Aliases:   []string{"ls"},
	ArgsUsage: "<zone>",
	Flags: []cli.Flag{
		internal.DNSProviderFlag,
	},
	Description: `
This command lists all domain name records managed by a 3rd party provider.

You must specify a zone (e.g. example.com) under which your domain name records
are listed. This zone must already be created on your provider account.
`,
	Action: func(c *cli.Context) error {
		zone := c.Args().First()
		if zone == "" {
			internal.ShowSubcommandHelpAndError(c, errors.New("no zone provided"))
		}

		var err error
		var provider gandi.Provider
		switch internal.DNSProvider {
		case "gandi":
			provider, err = gandiPkg.New()
			if err != nil {
				logrus.Fatal(err)
			}
		default:
			logrus.Fatalf("'%s' is not a supported DNS provider", internal.DNSProvider)
		}

		records, err := provider.GetRecords(c.Context, zone)
		if err != nil {
			logrus.Fatal(err)
		}

		tableCol := []string{"type", "name", "value", "TTL", "priority"}
		table := abraFormatter.CreateTable(tableCol)

		for _, record := range records {
			value := record.Value
			if len(record.Value) > 30 {
				value = fmt.Sprintf("%s...", record.Value[:30])
			}

			table.Append([]string{
				record.Type,
				record.Name,
				value,
				record.TTL.String(),
				strconv.Itoa(record.Priority),
			})
		}

		table.Render()

		return nil
	},
}
